@@ -5,23 +5,40 @@
 MOZ_APP_BASENAME=B2G
 MOZ_APP_VENDOR=Mozilla
 
-MOZ_APP_VERSION=16.0a1
+MOZ_APP_VERSION=43.0
 MOZ_APP_UA_NAME=Firefox
 
+MOZ_UA_OS_AGNOSTIC=1
+
+MOZ_B2G_VERSION=2.5.0.0-prerelease
+MOZ_B2G_OS_NAME=Boot2Gecko
+
 MOZ_BRANDING_DIRECTORY=b2g/branding/unofficial
 MOZ_OFFICIAL_BRANDING_DIRECTORY=b2g/branding/official
 # MOZ_APP_DISPLAYNAME is set by branding/configure.sh
 
-MOZ_SAFE_BROWSING=
-MOZ_SERVICES_SYNC=1
+MOZ_SAFE_BROWSING=1
+MOZ_SERVICES_COMMON=1
+MOZ_SERVICES_METRICS=1
 
 MOZ_WEBSMS_BACKEND=1
-MOZ_DISABLE_DOMCRYPTO=1
+MOZ_NO_SMART_CARDS=1
 MOZ_APP_STATIC_INI=1
+NSS_DISABLE_DBM=1
+MOZ_NO_EV_CERTS=1
+MOZ_DISABLE_EXPORT_JS=1
+
+MOZ_WEBSPEECH=1
+if test -n "$NIGHTLY_BUILD"; then
+MOZ_WEBSPEECH_MODELS=1
+MOZ_WEBSPEECH_POCKETSPHINX=1
+fi
+MOZ_WEBSPEECH_TEST_BACKEND=1
 
 if test "$OS_TARGET" = "Android"; then
 MOZ_CAPTURE=1
 MOZ_RAW=1
+MOZ_AUDIO_CHANNEL_MANAGER=1
 fi
 
 # use custom widget for html:select
@@ -31,11 +48,28 @@ if test "$LIBXUL_SDK"; then
 MOZ_XULRUNNER=1
 else
 MOZ_XULRUNNER=
-MOZ_PLACES=1
 fi
 
-MOZ_APP_ID={3c2e2abc-06d4-11e1-ac3b-374f68613e61}
-MOZ_EXTENSION_MANAGER=1
-ENABLE_MARIONETTE=1
+MOZ_MEDIA_NAVIGATOR=1
 
-MOZ_SYS_MSG=1
+MOZ_APP_ID={3c2e2abc-06d4-11e1-ac3b-374f68613e61}
+
+MOZ_TIME_MANAGER=1
+
+MOZ_SIMPLEPUSH=1
+MOZ_PAY=1
+MOZ_TOOLKIT_SEARCH=
+MOZ_PLACES=
+MOZ_B2G=1
+
+if test "$OS_TARGET" = "Android"; then
+MOZ_NUWA_PROCESS=1
+MOZ_B2G_LOADER=1
+MOZ_ENABLE_WARNINGS_AS_ERRORS=1
+fi
+
+MOZ_JSDOWNLOADS=1
+
+MOZ_BUNDLED_FONTS=1
+
+export JS_GC_SMALL_CHUNK_SIZE=1
